@@ -135,7 +135,7 @@ void CArrowTableIterator::reconstructRecordBatches()
                 ? std::stoi(metaData->value(metaData->FindKey("byteLength")))
                 : 16;
 
-          convertTimestampTZColumn(batchIdx, colIdx, field, columnArray, scale, byteLength, futureFields, futureColumns, needsRebuild);
+          convertTimestampTZColumn(batchIdx, colIdx, field, columnArray, scale, byteLength, futureFields, futureColumns, needsRebuild, m_timezone);
           break;
         }
 
@@ -303,25 +303,25 @@ void CArrowTableIterator::convertScaledFixedNumberColumnToDecimalColumn(
         case arrow::Type::type::INT8:
         {
           auto originalVal = std::static_pointer_cast<arrow::Int8Array>(columnArray)->Value(rowIdx);
-          val = arrow::Decimal128(0, originalVal);
+          val = arrow::Decimal128(originalVal);
           break;
         }
         case arrow::Type::type::INT16:
         {
           auto originalVal = std::static_pointer_cast<arrow::Int16Array>(columnArray)->Value(rowIdx);
-          val = arrow::Decimal128(0, originalVal);
+          val = arrow::Decimal128(originalVal);
           break;
         }
         case arrow::Type::type::INT32:
         {
           auto originalVal = std::static_pointer_cast<arrow::Int32Array>(columnArray)->Value(rowIdx);
-          val = arrow::Decimal128(0, originalVal);
+          val = arrow::Decimal128(originalVal);
           break;
         }
         case arrow::Type::type::INT64:
         {
           auto originalVal = std::static_pointer_cast<arrow::Int64Array>(columnArray)->Value(rowIdx);
-          val = arrow::Decimal128(0, originalVal);
+          val = arrow::Decimal128(originalVal);
           break;
         }
         default:
@@ -755,13 +755,43 @@ void CArrowTableIterator::convertTimestampColumn(
   }
   else
   {
+     bool has_overflow_to_downscale = false;
+     if (dt->id() == arrow::Type::type::STRUCT)
+     {
+         auto structArray = std::dynamic_pointer_cast<arrow::StructArray>(columnArray);
+         for(int64_t rowIdx = 0; rowIdx < columnArray->length(); rowIdx++)
+           {
+              if (columnArray->IsValid(rowIdx))
+              {
+                  int64_t epoch = std::static_pointer_cast<arrow::Int64Array>(
+                    structArray->GetFieldByName(sf::internal::FIELD_NAME_EPOCH))->Value(rowIdx);
+                  int32_t fraction = std::static_pointer_cast<arrow::Int32Array>(
+                    structArray->GetFieldByName(sf::internal::FIELD_NAME_FRACTION))->Value(rowIdx);
+                  int powTenSB4 = sf::internal::powTenSB4[9];
+                  if (epoch > (INT64_MAX / powTenSB4) || epoch < (INT64_MIN / powTenSB4))
+                  {
+                    if (fraction % 1000 != 0) {
+                        std::string errorInfo = Logger::formatString(
+                          "The total number of nanoseconds %d%d overflows int64 range. If you use a timestamp with "
+                          "the nanosecond part over 6-digits in the Snowflake database, the timestamp must be "
+                          "between '1677-09-21 00:12:43.145224192' and '2262-04-11 23:47:16.854775807' to not overflow."
+                          , epoch, fraction);
+                        throw std::overflow_error(errorInfo.c_str());
+                    } else {
+                        has_overflow_to_downscale = true;
+                    }
+                  }
+                }
+           }
+     }
+    auto timeUnit = has_overflow_to_downscale? arrow::TimeUnit::MICRO: arrow::TimeUnit::NANO;
     if (!timezone.empty())
     {
-      timeType = arrow::timestamp(arrow::TimeUnit::NANO, timezone);
+      timeType = arrow::timestamp(timeUnit, timezone);
     }
     else
     {
-      timeType = arrow::timestamp(arrow::TimeUnit::NANO);
+      timeType = arrow::timestamp(timeUnit);
     }
     tsField = std::make_shared<arrow::Field>(
       field->name(), timeType, field->nullable());
@@ -789,7 +819,13 @@ void CArrowTableIterator::convertTimestampColumn(
                 structArray->GetFieldByName(sf::internal::FIELD_NAME_EPOCH))->Value(rowIdx);
               int32_t fraction = std::static_pointer_cast<arrow::Int32Array>(
                 structArray->GetFieldByName(sf::internal::FIELD_NAME_FRACTION))->Value(rowIdx);
-              val = epoch * sf::internal::powTenSB4[9] + fraction;
+              if (has_overflow_to_downscale)
+              {
+                val = epoch * sf::internal::powTenSB4[6] + fraction / 1000;
+              } else
+              {
+                val = epoch * sf::internal::powTenSB4[9] + fraction;
+              }
             }
             break;
           default:
@@ -831,7 +867,8 @@ void CArrowTableIterator::convertTimestampTZColumn(
   const int byteLength,
   std::vector<std::shared_ptr<arrow::Field>>& futureFields,
   std::vector<std::shared_ptr<arrow::Array>>& futureColumns,
-  bool& needsRebuild
+  bool& needsRebuild,
+  const std::string timezone
 )
 {
   std::shared_ptr<arrow::Field> tsField;
@@ -848,28 +885,53 @@ void CArrowTableIterator::convertTimestampTZColumn(
 
   if (scale == 0)
   {
-    timeType = arrow::timestamp(arrow::TimeUnit::SECOND);
+    if (!timezone.empty())
+    {
+      timeType = arrow::timestamp(arrow::TimeUnit::SECOND, timezone);
+    }
+    else
+    {
+      timeType = arrow::timestamp(arrow::TimeUnit::SECOND);
+    }
     tsField = std::make_shared<arrow::Field>(
       field->name(), timeType, field->nullable());
   }
   else if (scale <= 3)
   {
-    timeType = arrow::timestamp(arrow::TimeUnit::MILLI);
-
+    if (!timezone.empty())
+    {
+      timeType = arrow::timestamp(arrow::TimeUnit::MILLI, timezone);
+    }
+    else
+    {
+      timeType = arrow::timestamp(arrow::TimeUnit::MILLI);
+    }
     tsField = std::make_shared<arrow::Field>(
       field->name(), timeType, field->nullable());
   }
   else if (scale <= 6)
   {
-    timeType = arrow::timestamp(arrow::TimeUnit::MICRO);
-
+    if (!timezone.empty())
+    {
+      timeType = arrow::timestamp(arrow::TimeUnit::MICRO, timezone);
+    }
+    else
+    {
+      timeType = arrow::timestamp(arrow::TimeUnit::MICRO);
+    }
     tsField = std::make_shared<arrow::Field>(
       field->name(), timeType, field->nullable());
   }
   else
   {
-    timeType = arrow::timestamp(arrow::TimeUnit::NANO);
-
+    if (!timezone.empty())
+    {
+      timeType = arrow::timestamp(arrow::TimeUnit::NANO, timezone);
+    }
+    else
+    {
+      timeType = arrow::timestamp(arrow::TimeUnit::NANO);
+    }
     tsField = std::make_shared<arrow::Field>(
       field->name(), timeType, field->nullable());
   }
